%-----------------------------------------------------
% Algoritmos geneticos - Bioingenieria I
%
% Historial de cambios:
%
% 20080430:
%    - agregada la variable 'elitista', que permite los
%      dos comportamientos.
%    - arreglados los redondeos en todas las funciones
%
% 20070501:
%    - arreglada la funcion mutaciones.m
%      (bug en el cambio del bit)
%    - agregadas las graficas del mejor fitness y 
%      del plano de soluciones a-b
%
% Version inicial: solucion de una senoidal de 
%                  amplitud (a) y frecuencia (b) desconocidas
%-----------------------------------------------------

clear all;
close all;

a=0.75;                       % Rango [0,1].
b=8;                          % Rango [0,10].
numindiv=100;                 % Numero de individuos de la poblacion
numprog=round(numindiv*0.2);  % Numero de progenitores a seleccionar
numcoef=2;
bits=8;                       % Numero de bits para la codificacion.
umbral_aptitud=0.99995;         % Umbral para finalizar por mejor fitness
pc=0.05;
pm=0.95;
rango1=[0,1];
rango2=[0,10];
elitista=0;					  % Bandera 0/1 para no/elitista

% Sistema a identificar

% t=0:0.01:1;
% m=length(t);
% y=a*sin(2*pi*b*t);

% sinc invertida
t=-1:0.01:1;
m=length(t);
a=0.75; b=5;
y=-a*sinc(2*pi*b*t);

factor_escala1=(2^bits-1)/(rango1(2)-rango1(1)); % Factor de escala para el parametro a.
factor_escala2=(2^bits-1)/(rango2(2)-rango2(1)); % Factor de escala para el parametro b. 
escalas=[factor_escala1 factor_escala2];

%---------- Inicializar poblacion ----------%
individuos=inicializa(numindiv,bits,escalas);

%---------- Evaluar fitnes ----------%
[aptitudes,mejor_individuo,mejor_aptitud]=fitness(individuos,escalas,bits,y);

% --------- Crear nuevas generaciones ----------%
generaciones=1;
contador=0;
maxgen=200;
apts=zeros(1,maxgen);
apts(generaciones)=0; %mejor_aptitud;
figure(1);
while and((mejor_aptitud < umbral_aptitud) , (generaciones < maxgen)),
    %---------- Generacion de nueva poblacion ----------%
    %-- Seleccion de progenitores por el metodo de competencias --%
    progenitores=progenitores_competencia(individuos,aptitudes,numprog);
    %-- Cruzas --%
    individuos=cruzas(progenitores,numindiv,bits,numcoef,pc);
    %-- Mutaciones --%
    individuos=mutaciones(individuos,bits,numcoef,pm);
    %-- Evaluar fitness --%
    [aptitudes,mejor_ind,mejor_apt,c1,c2]=fitness(individuos,escalas,bits,y);
    if mejor_apt>mejor_aptitud 
        mejor_aptitud=mejor_apt; 
        mejor_individuo=mejor_ind;
    end;
    generaciones=generaciones+1;
    disp('Generacion: '),disp(generaciones);
    
    % grafica de la aptitud en funcion de las generaciones
    apts(generaciones)=mejor_apt;
    subplot(1,2,1);
    plot(apts(2:generaciones));
    hold on;
    line([2 maxgen],[mejor_aptitud mejor_aptitud],'Color',[1 0 0],'LineStyle',':');
    hold off;
    axis([2 maxgen 0 1.2])
    title('Evolucion del mejor fitness');
    xlabel('nro. de generaciones');
    ylabel('fitness');
    legend('Mejor instantaneo','Mejor historico');
    
    % grafica del plano de coeficientes
    subplot(1,2,2);
    plot(c1,c2,'b*',a,b,'ro');
    axis([0 1 0 10])
    grid on;
    title('Espacio de soluciones');
    xlabel('a');
    ylabel('b');
    legend('Mejor instantanea','Solucion real');
    
    drawnow
end;

%----------Fin ----------%
if (elitista==0)
	[aptitudes,mejor_ind,mejor_apt,c1,c2]=fitness(individuos,escalas,bits,y);
end
ae=bin2dec(mejor_individuo(1:bits))*(1/escalas(1));
be=bin2dec(mejor_individuo(bits+1:2*bits))*(1/escalas(2));
disp('El fitness es: '),disp(mejor_aptitud),disp('Los coeficientes reales son: '),disp(a),disp(b),
disp('Los coeficientes estimados son: '),disp(ae),disp(be); 

% ye=ae*sin(2*pi*be*t);
ye=-ae*sinc(2*pi*be*t);

figure;
plot(t,y,'r'); hold on; plot(t,ye,'b');
title('Rojo: real, azu: estimada');
xlabel('tiempo [s]');
